import { db } from "$lib/server/db";
import * as table from "$lib/server/db/schema";
import { sha256 } from "@oslojs/crypto/sha2";
import { encodeBase32LowerCase, encodeHexLowerCase } from "@oslojs/encoding";
import { eq } from "drizzle-orm";

const DAY_IN_MS = 1000 * 60 * 60 * 24;

export const sessionCookieName = "auth-session";

export function generateSessionToken() {
  const bytes = crypto.getRandomValues(new Uint8Array(20));
  const token = encodeBase32LowerCase(bytes);
  return token;
}

/**
 * @param {string} token
 * @param {string} userId
 */
export async function createSession(token, userId) {
  const sessionId = encodeHexLowerCase(sha256(new TextEncoder().encode(token)));
  const session = {
    id: sessionId,
    userId,
    expiresAt: new Date(Date.now() + DAY_IN_MS * 30),
  };
  await db.insert(table.sessions).values(session);
  return session;
}

/** @param {string} token */
export async function validateSessionToken(token) {
  const sessionId = encodeHexLowerCase(sha256(new TextEncoder().encode(token)));
  const [result] = await db
    .select({
      // Adjust user table here to tweak returned data
      user: { id: table.users.id, username: table.users.username },
      session: table.sessions,
    })
    .from(table.sessions)
    .innerJoin(table.users, eq(table.sessions.userId, table.users.id))
    .where(eq(table.sessions.id, sessionId));

  if (!result) {
    return { session: null, user: null };
  }
  const { session, user } = result;

  const sessionExpired = Date.now() >= session.expiresAt.getTime();
  if (sessionExpired) {
    await db.delete(table.sessions).where(eq(table.sessions.id, session.id));
    return { session: null, user: null };
  }

  const renewSession = Date.now() >= session.expiresAt.getTime() - DAY_IN_MS * 15;
  if (renewSession) {
    session.expiresAt = new Date(Date.now() + DAY_IN_MS * 30);
    await db
      .update(table.sessions)
      .set({ expiresAt: session.expiresAt })
      .where(eq(table.sessions.id, session.id));
  }

  return { session, user };
}

/** @param {string} sessionId */
export async function invalidateSession(sessionId) {
  await db.delete(table.sessions).where(eq(table.sessions.id, sessionId));
}

/**
 * @param {import("@sveltejs/kit").RequestEvent} event
 * @param {string} token
 * @param {Date} expiresAt
 */
export function setSessionTokenCookie(event, token, expiresAt) {
  event.cookies.set(sessionCookieName, token, {
    expires: expiresAt,
    path: "/",
  });
}

/** @param {import("@sveltejs/kit").RequestEvent} event */
export function deleteSessionTokenCookie(event) {
  event.cookies.delete(sessionCookieName, {
    path: "/",
  });
}
